/******************************************************************************
 GETMACH.C - Network Host Use Monitor Machine Selection Utility

 This program contacts the NETUSE server and returns the hostname of a
 specific host on the network which meets given criteria.  If no host meets
 the criteria, no output is produced.  The -help option lists the possible
 selection criteria.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <netuse.h>


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     double l1,l2,l3;
     int users,console,i,modellist=0;
     long tmp;
     u_char machtype,modtype;
     char mname[132],buffer[512];

     l1=100.0; l2=100.0; l3=100.0;
     users=100; console=100;
     tmp=0;
     machtype=0; modtype=0;
     for (i=1; i<argc; i++) {
          strcpy(buffer,argv[i]);
          stoupper(buffer);
          if (!strcmp(buffer,"-L1")) {
               i++;
               if (sscanf(argv[i],"%lf",&l1)!=1) {
                    printf("Bad argument value (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-L2")) {
               i++;
               if (sscanf(argv[i],"%lf",&l2)!=1) {
                    printf("Bad argument value (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-L3")) {
               i++;
               if (sscanf(argv[i],"%lf",&l3)!=1) {
                    printf("Bad argument value (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-USERS")) {
               i++;
               if (sscanf(argv[i],"%d",&users)!=1) {
                    printf("Bad argument value (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-MODEL")) {
               i++;
               if (!(modtype=netStrToModel(argv[i]))) {
                    printf("Unknown model specifier (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-TMP")) {
               i++;
               if (sscanf(argv[i],"%ld",&tmp)!=1) {
                    printf("Bad argument value (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-CONSOLE")) {
               console=0;
               continue;
          }
          if (!strcmp(buffer,"-SUN")) {
               machtype=MACH_SUN;
               continue;
          }
          if (!strcmp(buffer,"-IBM")) {
               machtype=MACH_IBM_RS6000;
               continue;
          }
          if (!strcmp(buffer,"-APOLLO")) {
               machtype=MACH_APOLLO;
               continue;
          }
          if (!strcmp(buffer,"-DEC")) {
               machtype=MACH_DEC;
               continue;
          }
          if (!strcmp(buffer,"-HP")) {
               machtype=MACH_HP;
               continue;
          }
          if (!strcmp(buffer,"-HELP")) {
               printf("\nUsage: getmach [options]\n\n");
               printf("Options: -l1 <load>       Specify maximum 1-min load average\n");
               printf("         -l2 <load>       Specify maximum 5-min load average\n");
               printf("         -l3 <load>       Specify maximum 15-min load average\n");
               printf("         -users <nusers>  Specify maximum number of login sessions\n");
               printf("         -console         Specify no one logged into the display\n");
               printf("         -tmp <kbytes>    Specify minimum free space on /tmp\n");
               printf("         -model <model>   Specify model type\n");
               printf("         -sun             Request only Sun SparcStations\n");
               printf("         -ibm             Request only IBM RS/6000s\n");
               printf("         -dec             Request only DECstations\n");
               printf("         -apollo          Request only Apollos\n");
               printf("         -hp              Request only HP9000s\n\n");
               exit(0);
          }
     }
     netSetup();
     netuseFind(mname,l1,l2,l3,users,console,tmp,machtype,modtype);
     netShutdown();
     if (strlen(mname)) {
          printf("%s\n",mname);
          exit(0);
     }
     else exit(1);
}
