/******************************************************************************
 HOSTINFO.C - Netuse Host Use Monitor Host List Display Utility

 This utility contacts the NETUSE server and obtains a listing of machines
 which meet certain criteria.  These criteria currently include only the type
 of machine and the status (up/down).  The list is sorted by relative use
 (least-used first).  Only 10 list entries are normally given, but the -all
 option allows the entire list to be received and displayed.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <netuse.h>


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     u_char machtype,modeltype,mode;
     char buffer[512];
     int i;

     machtype=0; modeltype=0;
     mode=0;
     for (i=1; i<argc; i++) {
          strcpy(buffer,argv[i]);
          stoupper(buffer);
          if (!strcmp(buffer,"-SUN")) {
               machtype=MACH_SUN;
               continue;
          }
          if (!strcmp(buffer,"-IBM")) {
               machtype=MACH_IBM_RS6000;
               continue;
          }
          if (!strcmp(buffer,"-DEC")) {
               machtype=MACH_DEC;
               continue;
          }
          if (!strcmp(buffer,"-APOLLO")) {
               machtype=MACH_APOLLO;
               continue;
          }
          if (!strcmp(buffer,"-HP")) {
               machtype=MACH_HP;
               continue;
          }
          if (!strcmp(buffer,"-ANY")) {
               machtype=0;
               continue;
          }
          if (!strcmp(buffer,"-MODEL")) {
               ++i;
               if (!(modeltype=netStrToModel(argv[i]))) {
                    printf("Unknown model specifier (%s,%s).\n",argv[i-1],argv[i]);
                    --i;
               }
               continue;
          }
          if (!strcmp(buffer,"-ALL")) {
               mode=(mode & ~MODE_MASK) | MODE_ALL;
               continue;
          }
          if (!strcmp(buffer,"-DOWN")) {
               mode=(mode & ~MODE_MASK) | MODE_DOWN;
               continue;
          }
          if (!strcmp(buffer,"-MODELS")) {
               mode|=MODE_MODELS;
               continue;
          }
          if (!strcmp(buffer,"-NET")) {
               mode|=MODE_NET;
               continue;
          }
          if (!strcmp(buffer,"-HELP")) {
               printf("\nUsage: %s [options]\n\n",argv[0]);
               printf("Options: -sun             Only Sun SparcStations\n");
               printf("         -dec             Only DECstations\n");
               printf("         -ibm             Only IBM RS/6000s\n");
               printf("         -apollo          Only Apollos\n");
               printf("         -hp              Only HP9000s\n");
               printf("         -model <model>   Only machines of model type <model>\n");
               printf("         -down            Only down nodes\n");
               printf("         -any             Any type of machine\n");
               printf("         -all             Unlimited list length\n");
               printf("         -models          Show models instead of last report time\n");
               printf("         -net             Network server mode\n\n");
               exit(0);
          }
     }
     netSetup();
     netuseGetList(machtype,modeltype,mode);
     netShutdown();
}
