/******************************************************************************
 NETUSEADMIN.C - NETUSE Network Monitoring System Administrative Client

 This program provides a user interface to the remote NETUSE server, allowing
 various opertaions to be made on the host database.

 Lee Liming, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <netuse.h>
#include <parser.h>


#ifdef __STDC__
int get_command(char *buf)
#else
int get_command(buf)
char *buf;
#endif
{
     printf("\nnetuseadmin> ");
     gets(buf);
     return(0);
}


#ifdef __STDC__
void display_help(void)
#else
void display_help()
#endif
{
     printf("\nCommands:\n\n");
     printf("add       <vendor> <hostname> <model>  Add a new host to the database.\n");
     printf("delete    <hostname>           Delete a host from the database.\n");
     printf("list      [<vendor>]           List all hosts (or hosts from a given vendor)\n");
     printf("reload                         Load hosts from NETUSE hostfile.\n");
     printf("savestate                      Store host list in NETUSE hostfile.\n");
     printf("use       <filename>           Read additional commands from a file.\n");
     printf("\n?,help                         Display this help text.\n");
     printf("exit,quit                      Exit the program.\n");
     printf("\nVendor designations are currently 'dec', 'sun', 'apollo', 'hp', and 'ibm'.\n");
     printf("Hostnames may be either internet hostnames or IP addresses in standard format.\n");
     printf("All commands and arguments EXCEPT hostnames are case-insensitive.\n");
}


#ifdef __STDC__
int execute_command(char *cstr)
#else
int execute_command(cstr)
char *cstr;
#endif
{
     char command[128],buf[256];
     int mtype,modtype,mode,rv;
     FILE *inf;

     strcpy(buf,cstr);
     stoupper(strip(strip_leading(buf)));
     if (!strlen(buf)) return(0);
     if (!strcmp(textparam(buf,0,command),"QUIT") || !strcmp(command,"EXIT"))
          return(1);
     if (!strcmp(command,"?") || !strcmp(command,"HELP")) {
          display_help();
          return(0);
     }
     if (!strcmp(command,"SAVESTATE")) {
          if (!netuseSaveState()) printf("Host database has been stored.\n");
          else printf("An error occurred while storing the database.\n");
          return(0);
     }
     if (!strcmp(command,"RELOAD")) {
          if (!netuseLoadHosts()) printf("Host database has been reloaded.\n");
          else printf("An error occurred while loading the database.\n");
          return(0);
     }
     if (!strcmp(command,"DELETE")) {
          if (!strlen(textparam(cstr,1,command))) {
               printf("Missing hostname.  Use ? to view the command list.\n");
               return(0);
          }
          if (!netuseDelName(command)) printf("Delete command successful.\n");
          else printf("An error occurred during the delete operation.\n");
          return(0);
     }
     if (!strcmp(command,"ADD")) {
          if (!strlen(textparam(cstr,1,command))) {
               printf("Missing vendor type.  Use ? to view the command list.\n");
               return(0);
          }
          if (!(mtype=netStrToMach(command))) {
               printf("Invalid vendor type.  Vendors are DEC, IBM, Apollo, HP, and Sun.\n");
               return(0);
          }
          if (!strlen(textparam(cstr,3,command))) {
               printf("Missing model type.  Use ? to view the command list.\n");
               return(0);
          }
          if (!(modtype=netStrToModel(command))) {
               printf("Invalid model type.\n");
               return(0);
          }
          if (!strlen(textparam(cstr,2,command))) {
               printf("Missing hostname.  Use ? to view the command list.\n");
               return(0);
          }
          if (!(rv=netuseAddName(command,mtype,modtype))) 
               printf("Add command successful.\n");
          else printf("Error %d occurred during the add operation.\n",rv);
          return(0);
     }
     if (!strcmp(command,"USE")) {
          if (!strlen(textparam(cstr,1,command))) {
               printf("Missing filename.  Use ? to view the command list.\n");
               return(0);
          }
          if ((inf=fopen(command,"r"))==NULL) {
               printf("Unable to open %s for input.  USE command aborted...\n");
               return(0);
          }
          printf("\n");
          mtype=0;
          while (fgets(buf,sizeof(buf),inf) && !mtype) {
               buf[strlen(buf)-1]='\0';
               if (!strlen(buf) || (buf[0]=='#')) continue;
               printf("Executing: \"%s\"...\n",buf);
               mtype=execute_command(buf);
          }
          close(inf);
          return(0);
     }
     if (!strcmp(command,"LIST")) {
          mtype=0; mode=MODE_ALL;
          if (strlen(textparam(cstr,1,command))) {
               if (!(mtype=netStrToMach(command))) {
                    printf("Valid vendor types are DEC, IBM, Apollo, HP, and Sun.\n");
                    return(0);
               }
          }
          if (netuseGetList(mtype,0,mode)) {
               printf("Unable to display host list.\n");
               return(0);
          }
          return(0);
     }
     printf("\nType \"?\" or \"help\" to receive a list of valid commands.\n");
     return(0);
}


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     char command[256];
     int done;

     printf("\nNETUSE Administration Command Interpretor\n");
     netSetup();
     do {
          get_command(command);
          if (!feof(stdin)) done=execute_command(command);
     } while (!done && !feof(stdin));
     putchar('\n');
     netShutdown();
}
