/******************************************************************************
 CAENLAB.C - CAEN-specific heuristics for telling netused whether to run or 
             to quit.

 At CAEN, we have a file called /etc/hostconfig on all non-Apollo
 workstations.  If this file contains a line of the form "CLASS=LAB" in it, it
 means that the machine is in a public CAEN lab.  Apollos have a config file
 of a more complex format with similar information in it.

 We only run netused on lab machines, so this module contains code to tell
 netused whether it's supposed to run or not on this workstation.

 To enable this code, the macro USE_CAENLAB must be defined in 
 ../lib/config.h.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include "../lib/parser.h"
#include "caenlab.h"


#define CONFIGFILE     "/sys/node_data/etc/config"
#define HOSTCONFIGFILE "/etc/hostconfig"


#ifdef __STDC__
char *getfield(char *buf,int start,int stop,char *field)
#else
char *getfield(buf,start,stop,field)
char *buf;
int start,stop;
char *field;
#endif
{
     --start;
     --stop;
     if (strlen(buf)<(start+1)) field[0]='\0';
     else {
          strncpy(field,buf+start,(stop-start+1));
          field[stop-start+1]='\0';
     }
     strip(field);
     return(field);
}


#ifdef __STDC__
char *pwGetField(char *pwline,int n,char *buf)
#else
char *pwGetField(pwline,n,buf)
char *pwline;
int n;
char *buf;
#endif
{
     int i;
     char *p,*q;

     p=pwline;
     for (i=0; (i<n) && (*p!='\0'); p++) 
          if (*p==':') i++;
     for (q=buf; (*p!=':') && (*p!='\0'); )
          *(q++)=(*(p++));
     *q='\0';
     return(buf);
}


char *stolower(in)
char *in;
{
     char *p;

     for (p=in; *p!='\0'; p++)
          if (isupper(*p)) *p=tolower(*p);
}


#ifdef __STDC__
int apolloShouldRun(void)
#else
int apolloShouldRun()
#endif
{
     struct stat buf;
     int nameStart,nameEnd;
     int restStart,restEnd;
     int gotem,searchfield,print,theanswer;
     char name[20],rest[25];
     char inbuf[512],temp[256],*p,hostname[128];
     FILE *inf;
     
     theanswer=0;
     gethostname(hostname,sizeof(hostname));
     if ((p=strchr(hostname,'.'))!=NULL) *p='\0';
     if (stat(CONFIGFILE,&buf)) {
#ifdef TESTING
          printf("Apollo config file does not exist!\n");
#else
          logit("Apollo config file does not exist!");
#endif
          exit(1);
     }
     if ((inf=fopen(CONFIGFILE,"r"))==NULL) {
#ifdef TESTING
          printf("Unable to open the Apollo config file.\n");
#else
          logit("Unable to open the Apollo config file.");
#endif
          exit(1);
     }
     gotem=0;
     while (fgets(inbuf,sizeof(inbuf),inf)!=NULL) {
          if (!strlen(inbuf)) continue;
          inbuf[strlen(inbuf)]='\0';
          if (gotem<2) {
               if (!strncmp(inbuf,"#|<var> NODENAME",16)) {
                    nameStart=atoi(textparam(inbuf,2,temp));
                    nameEnd=atoi(textparam(inbuf,3,temp));
                    ++gotem;
               }
               if (!strncmp(inbuf,"#|<var> RESTRICTION",19)) {
                    restStart=atoi(textparam(inbuf,2,temp));
                    restEnd=atoi(textparam(inbuf,3,temp));
                    ++gotem;
               }
          }
          else {
               if ((inbuf[0]=='#') || !strlen(inbuf)) continue;
               print=0;
               getfield(inbuf,nameStart,nameEnd,name);
               for (p=name; (*p!='\0') && (*p=='/'); p++)
                    ;
               strcpy(name,p);
               stolower(name);
               getfield(inbuf,restStart,restEnd,rest);
               if (!strcmp(hostname,name) && !strcmp(rest,"L")) theanswer=1;
          }
     }
     if (gotem<2) {
#ifdef TESTING
          printf("Error in Apollo config file.\n");
#else
          logit("Error in Apollo config file.");
#endif
          exit(1);
     }
     fclose(inf);
     return(theanswer);
}


#ifdef __STDC__
int hostconfigShouldRun(void)
#else
int hostconfigShouldRun()
#endif
{
     char inbuf[132],*p,temp[132];
     FILE *inf;
     int theanswer,i;
     struct stat buf;
     
     theanswer=0;
     if (stat(HOSTCONFIGFILE,&buf)) {
#ifdef TESTING
          printf("hostconfig file does not exist!\n");
#else
          logit("hostconfig file does not exist!");
#endif
          exit(1);
     }
     if ((inf=fopen(HOSTCONFIGFILE,"r"))==NULL) {
#ifdef TESTING
          printf("Unable to open the hostconfig file.\n");
#else
          logit("Unable to open the hostconfig file.");
#endif
          exit(1);
     }
     while (fgets(inbuf,sizeof(inbuf),inf)!=NULL) {
          if (!strlen(inbuf)) continue;
          inbuf[strlen(inbuf)-1]='\0';
          if (!(p=strchr(inbuf,'='))) continue;
          *p='\0'; ++p;
          strip(strip_leading(inbuf));
          strip(strip_leading(p));
#ifdef TESTING
          printf("inbuf=\"%s\", p=\"%s\".\n",inbuf,p);
#endif
          if (!strcmp(inbuf,"CLASS")) {
               for (i=0; strlen(pwGetField(p,i,temp)) && !theanswer; i++)
                    if (!strcmp(temp,"LAB")) theanswer=1;
               break;
          }
     }
     fclose(inf);
     return(theanswer);
}


#ifdef __STDC__
int netusedShouldRun(void)
#else
int netusedShouldRun()
#endif
{
#ifdef apollo
     return(apolloShouldRun());
#else
     return(hostconfigShouldRun());
#endif
}


#ifdef TESTING

/* Test program for verifying functionality */

main()
{
     printf("netusedShouldRun() returns %d.\n",netusedShouldRun());
}


#endif
