/******************************************************************************
 This code gets the load average for a DEC or SUN.  (See below for extensions 
 to other platforms.)
 
 define mips for DEC
 define sun for sun. (the sun needs libkvm.a linked in)
 define testing for complilable program
 
 Hacked up by: Mike Neil on Oct 11, 1990.

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/


#if (defined(mips ) || defined(hpux))

/*
 * This code gets the load average for a DEC running Ultrix
 *
 * Hacked up by: Mike Neil from the xload sources.
 */


#include <stdio.h>
#include <nlist.h>


extern long lseek();
extern void exit();

static struct nlist namelist[] = {	    
#ifdef hpux
    {"avenrun"},
#else
    {"_avenrun"},
#endif
    {0}
};

long loadavg_seek;

#ifdef __STDC__
void InitGetLoad(void)
#else
void InitGetLoad()
#endif
{
	extern void nlist();

#ifdef hpux
	nlist( "/hp-ux", namelist);
#else
	nlist( "/vmunix", namelist);
#endif
	if (namelist[0].n_type == 0 || namelist[0].n_value == 0) {
    /* error getting name index */
  	exit(-1);
	}
	loadavg_seek = namelist[0].n_value;

}

#ifdef __STDC__
double GetLoadPoint(double *l1,double *l2,double *l3)
#else
double GetLoadPoint(l1,l2,l3)
double *l1,*l2,*l3;
#endif
{
	static kmem;

	kmem = open("/dev/kmem", 000		);
	if (kmem < 0)
	{
	  /* Error opening /dev/kmem */
	}
	
	(void) lseek(kmem, loadavg_seek, 0);
	{
#ifdef hpux
		read(kmem, (char *)l1, sizeof(double));
		read(kmem, (char *)l2, sizeof(double));
		read(kmem, (char *)l3, sizeof(double));
#else
		int temp;
		(void) read(kmem, (char *)&temp, sizeof(int));
		*l1 = 	(((double)(temp))/(1<<	8));
		(void) read(kmem, (char *)&temp, sizeof(int));
		*l2 = 	(((double)(temp))/(1<<	8));
		(void) read(kmem, (char *)&temp, sizeof(int));
		*l3 = 	(((double)(temp))/(1<<	8));
#endif
	}
  close(kmem);	
	
	return(*l1);
}
#endif


#ifdef sun
/*
 * This code gets the load average for a SUN running 4.x.x
 *
 * Hacked up by: Mike Neil from the w sources.
 */

#include <stdio.h>
#include <nlist.h>
#include <sys/stat.h>
#include <sys/proc.h>
#include <kvm.h>
#include <fcntl.h>


long	avenrun[3];
kvm_t	*kd;			/* kernel descriptor for Kernel VM calls */
char	*prog;			/* pointer to invocation name */

struct	nlist nl[] = {
	{ "_avenrun" },
#define	X_AVENRUN	0
	{ "" },
};


/*
 * readsym: get the value of a symbol from the namelist.
 * exit if error.
 */
readsym(n, b, s)
int n;
char *b;
int s;
{
	if (nl[n].n_type == 0) {
		fprintf(stderr, "%s: '%s' not in namelist\n",
		    prog, nl[n].n_name);
		return(-1);
	}
	if (kvm_read(kd, nl[n].n_value, b, s) != s) {
		fprintf(stderr, "%s: kernel read error\n", prog);
		return(-1);
	}
}

#ifdef __STDC__
void InitGetLoad(void)
#else
void InitGetLoad()
#endif
{
	/*Just a stub */
}

#ifdef __STDC__
double GetLoadPoint(double *l1,double *l2,double *l3)
#else
double GetLoadPoint(l1,l2,l3)
double *l1,*l2,*l3;
#endif
{
	double loadavg;

			if ((kd = kvm_open(NULL, NULL, NULL, O_RDONLY, prog)) == NULL) {
				return(-1);
			}
			if (kvm_nlist(kd, nl) != 0) {
				fprintf(stderr, "%s: symbols missing from namelist\n", prog);
				return(-1);
			}
	
			readsym(X_AVENRUN, avenrun, sizeof(avenrun));
			
			*l1 = (double)avenrun[0]/FSCALE;
			*l2 = (double)avenrun[1]/FSCALE;
			*l3 = (double)avenrun[2]/FSCALE;

                        kvm_close(kd);
			return(*l1);
}
#endif

#ifdef _AIX

/*
 * This version of this module (getload.c) is specifically for the IBM
 * RS/6000 AIX 3.1 platform.
 *
 *  Charley Kline, University of Illinois Computing Services
 *  c-kline@uiuc.edu
 *
 * Modified for use with 1, 5, and 15 minute averages by Michael Neil and
 * Lee Liming.
 */


#include <stdio.h>
#include <sys/sysinfo.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <math.h>
#include <nlist.h>


struct nlist kernelnames[] = {{"sysinfo", 0, 0, 0, 0, 0},{NULL, 0, 0, 0, 0, 0}};


#ifdef __STDC__
void InitGetLoad(void)
#else
void InitGetLoad()
#endif
{
	/*Just a stub */
}


#ifdef __STDC__
double GetLoadPoint(double *l1,double *l2,double *l3)
#else
double GetLoadPoint(l1,l2,l3)
double *l1,*l2,*l3;
#endif
{
    static double avenrun=0.0,avenrun5=0.0,avenrun15=0.0;
    double loadav;
    struct sysinfo si;
    static int rq_old = 0, ro_old = 0;
    static initted = 0;
    static int fd;
    double multiplier;
    double t;

    /*
    ** Do stuff we only need to do once per invocation, like opening
    ** the kmem file and fetching the parts of the symbol table.
    */
		if (!initted) {
				initted = 1;
				knlist(kernelnames, 1, sizeof (struct nlist));
				fd = open("/dev/kmem", O_RDONLY);
				if (fd < 0) {
						perror("kmem");
						exit(1);
				}
		}
		
		/*
    ** Get the system info structure from the running kernel.
    */
    lseek(fd, kernelnames[0].n_value, SEEK_SET);
    read(fd, &si, sizeof (struct sysinfo));

    /*
    ** AIX doesn't keep the load average variables in the kernel; all
    ** we can get is the current number of runnable processes by
    ** observing the difference between the runque and runocc values
    ** in sysinfo, and dividing. Having done this, however, we can apply
    ** a TENEX-style exponential time-average to it by computing an
    ** averaging multiplier based on the sampling interval. This is then
    ** used to factor in the current number of runnable processes to our
    ** running load average. The result "looks right" when observed in
    ** conjunction with the process table and user activity.
    **
    ** We subtract one from the number of running processes given us by
    ** the kernel because for some reason AIX always calls one of the
    ** kprocs "runnable" even though it uses very little CPU. Subtracting
    ** this out gives a load average near zero when the machine is mostly
    ** idle, which is more familiar to those of us who are used to
    ** bsd-style load averages.                   /cvk
    */
    
    t = (double)(si.runocc - ro_old);
    loadav = (double)(si.runque - rq_old) / t - 1.0;
    rq_old = si.runque;
    ro_old = si.runocc;
    multiplier = exp(-t/60.);
    avenrun = multiplier * avenrun + (1.0 - multiplier) * loadav;

    multiplier = exp(-t/300.);
    avenrun5 = multiplier * avenrun5 + (1.0 - multiplier) * loadav;

    multiplier = exp(-t/900.);
    avenrun15 = multiplier * avenrun15 + (1.0 - multiplier) * loadav;
    
/*
    printf("%d %d %f %f %f %f %f\n", si.runque, si.runocc, t, loadav, avenrun,
    avenrun5, avenrun15);
*/
    *l1=avenrun; *l2=avenrun5; *l3=avenrun15;
    return(*l1);
    }

/*
void main()
{
		int	i;
		
		for (i = 0; i < 10; i++) {
				GetLoadPoint();
				sleep(10);
		}
}
*/

#endif


#ifdef apollo

/* 
  This code hacked up by Lee Liming, 10/11/91, from the Apollo uptime sources.
*/

#include <stdio.h>
#include <sys/time.h>


typedef long proc1_$loadav_t[3];

void proc1_$get_loadav();


#ifdef __STDC__
void InitGetLoad(void)
#else
void InitGetLoad()
#endif
{
}


#ifdef __STDC__
double GetLoadPoint(double *l1,double *l2,double *l3)
#else
double GetLoadPoint(l1,l2,l3)
double *l1,*l2,*l3;
#endif
{
     proc1_$loadav_t avenrun;
     int i;

     proc1_$get_loadav(avenrun);
     *l1=avenrun[0]/65536.;
     *l2=avenrun[1]/65536.;
     *l3=avenrun[2]/65536.;
     return(*l1);
}

#endif /* apollo */


#ifdef testing
main()
{
     double l1,l2,l3,rv;

     printf("Starting...\n"); fflush(stdout);
     InitGetLoad();
     printf("Between...\n"); fflush(stdout);
     rv=GetLoadPoint(&l1,&l2,&l3);
     printf("rv=%0.2lf l1=%0.2lf l2=%0.2lf l3=%0.2lf\n",rv,l1,l2,l3);
}
#endif
