/******************************************************************************
 NETUSE.C - Network Host Use Monitor Server

 This program acts as a server, receiving periodic information from network
 hosts concerning their current states.  It also accepts requests for 
 information concerning these hosts, and makes the information available to
 client machines.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <setjmp.h>
#include <pwd.h>
#include <errno.h>
#if (defined(_AIX) || defined(apollo))
#include <sys/ioctl.h>
#else
#include <sys/termios.h>
#endif

#include "../lib/protocol.h"
#include "../lib/netuse.h"
#include "../lib/config.h"
#include "../lib/variable.h"


/* #define DEBUG */


extern long lTime;
extern VARTABLE variables;

int sock;
struct sockaddr_in myname;
jmp_buf jumpTimeout;
long lastCheck=0,lastSave=0,netuseStartTime;


#ifdef __STDC__
int packetSend(USEREC *msg,struct sockaddr_in *dest)
#else
int packetSend(msg,dest)
USEREC *msg;
struct sockaddr_in *dest;
#endif
{
     u_char chk;
     int rv,count,i;
     PACKET out;

     msg->load1=htons(msg->load1);
     msg->load2=htons(msg->load2);
     msg->load3=htons(msg->load3);
     msg->users=htons(msg->users);
     msg->console=htons(msg->console);
     msg->tmp=htonl(msg->tmp);
     msg->uid=htons(msg->uid);
     bcopy(msg,out,sizeof(PACKET));
     for (chk=0,i=1; i<PACKET_SIZE; i++)
          chk=(chk+out[i]) % 0xFF;
     out[0]=chk;
#ifdef DEBUG
     printf("Server sending:\n");
     printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
            msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
            ntohs(msg->load2),ntohs(msg->load3));
     printf("Users=%04x Console=%04x Tmp=%08x Mach=%2x Model=%02x Chksum=%02x\n",
            ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
            msg->machine,msg->model,out[0]);
     printf("Data:");
     for (i=0; i<PACKET_SIZE; i++) {
          if (i && !(i % 16)) printf("\n     ");
          printf(" %02x",out[i]);
     }
     printf("\n");
#endif
     count=0;
     do {
          rv=sendto(sock,out,PACKET_SIZE,0,dest,sizeof(struct sockaddr_in));
          if (rv==(-1)) {
               perror("sendto");
               if (++count>SEND_RETRIES) return(RV_nBADSEND);
          }
     } while (rv!=PACKET_SIZE);
     return(RV_nOK);
}


#ifdef __STDC__
int AlarmHandler(int sig)
#else
int AlarmHandler(sig)
int sig;
#endif
{
     longjmp(jumpTimeout,1);
     return(0);
}


#ifdef __STDC__
int packetReceiveT(USEREC *msg,struct sockaddr_in *src)
#else
int packetReceiveT(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     u_char chk;
     int i;
     PACKET in;

     do {
          signal(SIGALRM,AlarmHandler);
          if (setjmp(jumpTimeout)!=0) return(RV_nTIMEOUT);
          alarm(TIMEOUT);
          i=sizeof(struct sockaddr_in);
          if (recvfrom(sock,in,PACKET_SIZE,0,src,&i)==(-1)) {
               perror("recvfrom");
               return(RV_nBADRECV);
          }
          alarm(0);
          signal(SIGALRM,SIG_DFL);
          bcopy(in,msg,sizeof(PACKET));
#ifdef DEBUG
          printf("Server received:\n");
          printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3));
          printf("Users=%04x Console=%04x Tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
                 ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
                 msg->machine,msg->model,in[0]);
          printf("Data:");
          for (i=0; i<PACKET_SIZE; i++) {
               if (i && !(i % 16)) printf("\n     ");
               printf(" %02x",in[i]);
          }
          printf("\n");
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
     } while (chk!=msg->chksum);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     return(RV_nOK);
}


#ifdef __STDC__
int packetReceive(USEREC *msg,struct sockaddr_in *src)
#else
int packetReceive(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     u_char chk;
     int i;
     PACKET in;

     do {
          i=sizeof(struct sockaddr_in);
          if (recvfrom(sock,in,PACKET_SIZE,0,src,&i)==(-1)) {
               perror("recvfrom");
               return(RV_nBADRECV);
          }
          bcopy(in,msg,sizeof(PACKET));
#ifdef DEBUG
          printf("Server received:\n");
          printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3));
          printf("Users=%04x Console=%04x Tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
                 ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
                 msg->machine,msg->model,in[0]);
          printf("PACKET_SIZE=%d\n",PACKET_SIZE);
          printf("Data:");
          for (i=0; i<PACKET_SIZE; i++) {
               if (i && !(i % 16)) printf("\n     ");
               printf(" %02x",in[i]);
          }
          printf("\n");
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
#ifdef DEBUG
          if (chk!=msg->chksum)
               printf("Bad checksum! %2x should be %2x\n",msg->chksum,chk);
#endif
     } while (chk!=msg->chksum);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     return(RV_nOK);
}


#ifdef __STDC__
int netSetup(void)
#else
int netSetup()
#endif
{
     struct servent *servptr;
     int portno;

     servptr=getservbyname(NETUSE_SERVICE,NULL);
     if (servptr==NULL) portno=NETUSE_PORT;
     else portno=servptr->s_port;
     sock=socket(AF_INET,SOCK_DGRAM,0);
     if (sock<0) {
          perror("Acquiring a socket");
          return(1);
     }
     myname.sin_family=AF_INET;
     myname.sin_addr.s_addr=INADDR_ANY;
     myname.sin_port=portno;
     if (bind(sock,&myname,sizeof(myname))) {
          perror("Binding socket");
          return(1);
     }
     return(0);
}


#ifdef __STDC__
void netShutdown(void)
#else
void netShutdown()
#endif
{
     close(sock);
}


#ifdef __STDC__
int handleReport(USEREC *msg,struct sockaddr_in *src)
#else
int handleReport(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     char hostname[255];
     VARENTRY *ventry;

     ventry=varAllocate();
     bcopy(&(src->sin_addr.s_addr),&(ventry->ip_addr),sizeof(ventry->ip_addr));
     if (netGetHostname(ventry->ip_addr,hostname)==NULL) return(-1);
     ventry->name=(char *)malloc(strlen(hostname)+1);
     strcpy(ventry->name,hostname);
     ventry->l1=1.0*msg->load1/100;
     ventry->l2=1.0*msg->load2/100;
     ventry->l3=1.0*msg->load3/100;
     ventry->users=msg->users;
     ventry->console=msg->console;
     ventry->tmp=msg->tmp;
     ventry->machine=msg->machine;
     ventry->model=msg->model;
     varSet(ventry);
     varFree(ventry);
#ifdef ACKNOWLEDGE
     msg.opcode=OP_NOP;
     msg.ack=RV_ACK;
     packetSend(msg,src);
#endif
     return(0);
}


/*****************************************************************************
 void logit(msg)

 This function logs a message in the (previously opened) logfile.  Note that
 fflush() is used to ensure that the message gets logged to disk.
*****************************************************************************/

#ifdef __STDC__
void logit(char *msg)
#else
void logit(msg)
char *msg;
#endif
{
     char atime[64];
     struct timeval tv;
     struct timezone tz;
     int retries;
     FILE *logf;

     gettimeofday(&tv,&tz);
     strcpy(atime,asctime(localtime(&(tv.tv_sec))));
     atime[strlen(atime)-1]='\0';
     for (retries=0; (retries<5) && ((logf=fopen(NETUSELOG,"a"))==NULL); ++retries)
          sleep(5);
     if (retries==5) {
          fprintf(stderr,"Unable to open log file %s for update.\n",NETUSELOG);
          return;
     }
     fprintf(logf,"%s -- %s\n",atime,msg);
     fclose(logf);
     chmod(NETUSELOG,0644);
}


#ifdef __STDC__
void logQuery(VARENTRY *template,u_short uid,struct sockaddr_in *src)
#else
void logQuery(template,uid,src)
VARENTRY *template;
u_short uid;
struct sockaddr_in *src;
#endif
{
     char it[132],source[80],mtype[20],modtype[16],id[30];
     struct passwd *pwptr;

     netGetHostname(src->sin_addr.s_addr,source);
     netGetMachType(template->machine,mtype);
     netGetModelType(template->model,modtype);
     pwptr=getpwuid(uid);
     if (pwptr!=NULL) strcpy(id,pwptr->pw_name);
     else sprintf(id,"%hu",uid);
     sprintf(it,"getmach: (%s,%s) <- %s %0.2f %0.2f %0.2f %hd %s %ld %s %s",
             id,source,(strlen(template->name) ? template->name : "(none)"),
             template->l1,template->l2,template->l3,template->users,
             (template->console ? "YES" : "NO"),template->tmp,mtype,modtype);
     logit(it);
}


#ifdef __STDC__
void logList(u_char opcode,u_char machine,u_char model,u_short uid,struct sockaddr_in *src)
#else
void logList(opcode,machine,model,uid,src)
u_char opcode,machine,model;
u_short uid;
struct sockaddr_in *src;
#endif
{
     char it[132],source[80],type[32],mtype[20],modtype[16],id[30];
     struct passwd *pwptr;

     switch (opcode) {
          case OP_GETLIST:
          case OP_GETMLIST:
               strcpy(type,"ALL");
               break;
          case OP_DISPLIST:
          case OP_DISPMLIST:
               strcpy(type,"TEN");
               break;
          case OP_GETDOWN:
          case OP_GETMDOWN:
               strcpy(type,"DOWN");
               break;
          default:
               strcpy(type,"ERROR!");
               break;
     }
     netGetHostname(src->sin_addr.s_addr,source);
     netGetMachType(machine,mtype);
     netGetModelType(model,modtype);
     pwptr=getpwuid(uid);
     if (pwptr!=NULL) strcpy(id,pwptr->pw_name);
     else sprintf(id,"%hu",uid);
     sprintf(it,"hostinfo: (%s,%s) %s %s %s",id,source,type,mtype,modtype);
     logit(it);
}


#ifdef __STDC__
int handleQuery(USEREC *msg,struct sockaddr_in *src)
#else
int handleQuery(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     VARENTRY *ventry,*vptr;

     ventry=varAllocate();
     ventry->l1=1.0*msg->load1/100;
     ventry->l2=1.0*msg->load2/100;
     ventry->l3=1.0*msg->load3/100;
     ventry->users=msg->users;
     ventry->console=msg->console;
     ventry->tmp=msg->tmp;
     ventry->machine=msg->machine;
     ventry->model=msg->model;
     vptr=varFindMatch(ventry);
     if (vptr==NULL) {
          ventry->name=(char *)malloc(1);
          ventry->name[0]='\0';
#ifdef LOGREQUESTS
          logQuery(ventry,msg->uid,src);
#endif
          msg->retcode=RV_NOMATCH;
          packetSend(msg,src);
     }
     else {
          ventry->name=(char *)malloc(strlen(vptr->name)+1);
          strcpy(ventry->name,vptr->name);
#ifdef LOGREQUESTS
          logQuery(ventry,msg->uid,src);
#endif
          msg->ipaddr=netGetAddress(ventry->name);
          msg->retcode=RV_OK;
          packetSend(msg,src);
     }
     varFree(ventry);
     return(0);
}


#ifdef __STDC__
int handleList(USEREC *msg,struct sockaddr_in *src)
#else
int handleList(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     int tcpsock;
     FILE *outf;

     tcpsock=socket(AF_INET,SOCK_STREAM,0);
     if (tcpsock<0) {
          perror("Acquiring TCP socket");
          return(-1);
     }
     src->sin_port=htons(msg->portno);
#ifdef DEBUG
     printf("About to connect()...\n");
#endif
     if (connect(tcpsock,src,sizeof(struct sockaddr_in))<0) {
          perror("Connecting to client's TCP port");
          return(-1);
     }
#ifdef DEBUG
     printf("Connection established...\n");
#endif
     if ((outf=fdopen(tcpsock,"w"))==NULL) {
          perror("fdopen");
          return(-1);
     }
#ifdef DEBUG
     printf("fdopen() established...\n");
#endif
     switch (msg->opcode) {
          case OP_GETLIST:
          case OP_GETDOWN:
          case OP_DISPLIST:
               treePrintList(outf,msg->machine,msg->model,msg->opcode);
               break;
          case OP_GETMLIST:
          case OP_GETMDOWN:
          case OP_DISPMLIST:
               treePrintListWithModels(outf,msg->machine,msg->model,msg->opcode);
               break;
          default:
#ifdef DEBUG
               printf("Don't know what type of list to print!\n");
#endif
               treePrintList(outf,msg->machine,msg->model,msg->opcode);
               break;
     }
#ifdef DEBUG
     printf("After treePrintList()...\n");
#endif
     fflush(outf);
     fclose(outf);
     close(tcpsock);
#ifdef LOGREQUESTS
     logList(msg->opcode,msg->machine,msg->model,msg->uid,src);
#endif
     return(0);
}


#ifdef __STDC__
int handleHostAdd(USEREC *msg,struct sockaddr_in *src)
#else
int handleHostAdd(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     char hostname[255],temp[132];
     VARENTRY *ventry;

     ventry=varAllocate();
     ventry->ip_addr=msg->ipaddr;
     if (netGetHostname(ventry->ip_addr,hostname)==NULL) return(-1);
     ventry->name=(char *)malloc(strlen(hostname)+1);
     strcpy(ventry->name,hostname);
     ventry->machine=msg->machine;
     ventry->model=msg->model;
     varAdd(ventry);
     varFree(ventry);
     sprintf(temp,"Added host %s of type (%d,%d).",hostname,msg->machine,msg->model);
     logit(temp);
     msg->retcode=RV_OK;
     msg->ack=RV_ACK;
     packetSend(msg,src);
     return(0);
}


#ifdef __STDC__
int handleHostDelete(USEREC *msg,struct sockaddr_in *src)
#else
int handleHostDelete(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     char hostname[255],temp[255];
     VARENTRY *ventry;

     ventry=varAllocate();
     ventry->ip_addr=msg->ipaddr;
     if (netGetHostname(ventry->ip_addr,hostname)==NULL) {
          msg->retcode=RV_NOMATCH;
          packetSend(msg,src);
          return(0);
     }
     ventry->name=(char *)malloc(strlen(hostname)+1);
     strcpy(ventry->name,hostname);
     varDelete(ventry);
     varFree(ventry);
     msg->retcode=RV_OK;
     msg->ack=RV_ACK;
     packetSend(msg,src);
     sprintf(temp,"Deleted host %s",hostname);
     logit(temp);
     return(0);
}


#ifdef __STDC__
int handleStateSave(USEREC *msg,struct sockaddr_in *src)
#else
int handleStateSave(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     msg->retcode=varSaveState();
     msg->ack=RV_ACK;
     packetSend(msg,src);
     return(0);
}


#ifdef __STDC__
int handleHostLoad(USEREC *msg,struct sockaddr_in *src)
#else
int handleHostLoad(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     msg->retcode=varLoadHosts();
     msg->ack=RV_ACK;
     packetSend(msg,src);
     return(0);
}


#ifdef __STDC__
void WakeTheDead(void)
#else
void WakeTheDead()
#endif
{
     VARENTRY *vptr,*vprev;
     int i,socket;
     struct servent *servptr;
     struct sockaddr_in dest;
     u_long ip;
     char buf[128];
     long lastrep;

#ifdef DEBUG
     logit("Checking for dead daemons.  (Can daemons die?)");
#endif
     servptr=getservbyname(NETUSED_SERVICE,NULL);
     if (servptr==NULL) dest.sin_port=NETUSED_PORT;
     else dest.sin_port=servptr->s_port;
     dest.sin_family=AF_INET;
     for (i=0; i<VAR_ENTRIES; i++) {
          vprev=NULL;
          for (vptr=variables[i]; vptr!=NULL; vptr=vptr->next) {
               if (!vptr->logtime) lastrep=netuseStartTime;
               else lastrep=vptr->logtime;
               if ((time(NULL)-lastrep)>=GIVEUPTIME*60) {
                    sprintf(buf,"Retiring host %s.",vptr->name);
                    logit(buf);
                    if (vprev==NULL) variables[i]=vptr->next;
                    else vprev->next=vptr->next;
                    varFree(vptr);
                    sprintf(buf,"Host %s has been retired.",vptr->name);
                    logit(buf);
                    if (vprev==NULL) {
                         if ((vptr=variables[i])==NULL) break;
                    }
                    else vptr=vprev;
                    continue;      /* vprev is already correct for next loop */
               }
               if ((time(NULL)-lastrep)>=DOWNTIME*60) {
                    if (ip=netGetAddress(vptr->name)) {
                         dest.sin_addr.s_addr=ip;
#ifdef LOGWAKEUPS
                         sprintf(buf,"Yo!  (Waking up %s)...",vptr->name);
                         logit(buf);
#endif
                         if (sendto(sock,"Wakeup!",9,0,&dest,sizeof(dest))==(-1)) {
                              sprintf(buf,"Error %d from sendto() in WakeTheDead().\n",
                                          errno);
                              logit(buf);
                         }
                    }
               }
               vprev=vptr;
          }
     }
}


#ifdef __STDC__
int netGetRequest(USEREC *msg,struct sockaddr_in *src)
#else
int netGetRequest(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     return(packetReceive(msg,src));
}


#ifdef __STDC__
int netHandleRequest(USEREC *msg,struct sockaddr_in *src)
#else
int netHandleRequest(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     switch (msg->opcode) {
          case OP_STATUS:
               handleReport(msg,src);
               break;
          case OP_QUERY:
               handleQuery(msg,src);
               break;
          case OP_GETLIST:
          case OP_GETDOWN:
          case OP_DISPLIST:
          case OP_GETMLIST:
          case OP_GETMDOWN:
          case OP_DISPMLIST:
               handleList(msg,src);
               break;
          case OP_ADDHOST:
               handleHostAdd(msg,src);
               break;
          case OP_DELHOST:
               handleHostDelete(msg,src);
               break;
          case OP_SAVESTATE:
               handleStateSave(msg,src);
               break;
          case OP_LOADHOSTS:
               handleHostLoad(msg,src);
               break;
/*        case OP_TEXTSTATUS:
               handleTextReport(msg,src,0);
               break;
          case OP_TEXTSTATUS2:
               handleTextReport(msg,src,1);
               break; */
          default:
               printf("Unknown opcode received...\n");
               break;
     }
     if ((time(0)-lastCheck)>DOWNTIME*60) {
          WakeTheDead();
          lastCheck=time(0);
     }
     if ((time(0)-lastSave)>DOWNTIME*60) {
          varSaveState();
          lastSave=time(0);
     }
     return(0);
}


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     USEREC msg;
     struct sockaddr_in src;

     if (fork()) exit(0);
     varInit();
     netuseStartTime=lastCheck=lastSave=time(0);
     while (netSetup()) 
          sleep(5);
     for ( ; ; ) {
          netGetRequest(&msg,&src);
#ifdef DEBUG
          printf("About to call netHandleRequest()...\n");
#endif
          netHandleRequest(&msg,&src);
#ifdef DEBUG
          printf("Back from netHandleRequest().\n");
#endif
     }
     netShutdown();
     varFreeTable();
}
