/******************************************************************************
 NETUSED.C - Network Host Use Monitor Daemon

 This program runs on a host machine on a network and periodically reports
 information about the machine to the NETUSE server.  Currently, the
 information sent includes the load averages, number of active users, use of
 the console, machine type, and the kilobytes of free space on the /tmp
 filesystem.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <signal.h>
#include <setjmp.h>
#include <sys/time.h>
#include <sys/errno.h>
#ifdef hpux
#include <unistd.h>
#endif
#if (defined(_AIX) || defined(apollo))
#include <sys/ioctl.h>
#else
#include <sys/termios.h>
#endif
#ifdef _AIX
#include <sys/select.h>
#endif
#include "../lib/protocol.h"
#include "../lib/netuse.h"
#include "../lib/config.h"
#include "caenlab.h"


/* #define DEBUG */


extern errno;

int sock;
struct sockaddr_in servername,clientname;
jmp_buf jumpTimeout;
u_char modelMine=0;

int  netSetup();
void netShutdown();


/*****************************************************************************
 void logit(msg)

 This function logs a message in the logfile.
*****************************************************************************/

#ifdef __STDC__
void logit(char *msg)
#else
void logit(msg)
char *msg;
#endif
{
     char atime[64],buf[132];
     struct timeval tv;
     struct timezone tz;
     int retries;
     FILE *logf;

     gettimeofday(&tv,&tz);
     strcpy(atime,asctime(localtime(&(tv.tv_sec))));
     atime[strlen(atime)-1]='\0';
     for (retries=0; (retries<5) && ((logf=fopen(NETUSEDLOG,"a"))==NULL); ++retries)
          sleep(5);
     if (retries==5) {
          fprintf(stderr,"Unable to open log file %s for update.\n",NETUSEDLOG);
          return;
     }
     fprintf(logf,"%s -- %s\n",atime,msg);
     fclose(logf);
     chmod(NETUSEDLOG,0644);
}


#ifdef __STDC__
int packetSend(USEREC *msg)
#else
int packetSend(msg)
USEREC *msg;
#endif
{
     u_char chk;
     int rv,count,i;
     PACKET out;
     char buf[132];

     if (netSetup()) return(RV_nBADSEND);
     msg->load1=htons(msg->load1);
     msg->load2=htons(msg->load2);
     msg->load3=htons(msg->load3);
     msg->users=htons(msg->users);
     msg->console=htons(msg->console);
     msg->tmp=htonl(msg->tmp);
     msg->uid=htons(msg->uid);
     bcopy(msg,out,sizeof(PACKET));
     for (chk=0,i=1; i<PACKET_SIZE; i++)
          chk=(chk+out[i]) % 0xFF;
     out[0]=chk;
#ifdef DEBUG
     logit("Client sending:");
     sprintf(buf,"Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x Users=%04x Console=%04x /tmp=%08x Mach=%02x Model=%02x Chksum=%02x",
            msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
            ntohs(msg->load2),ntohs(msg->load3),ntohs(msg->users),
            ntohs(msg->console),ntohl(msg->tmp),msg->machine,msg->model,out[0]);
     logit(buf);
#endif
     count=0;
     do {
          rv=sendto(sock,out,PACKET_SIZE,0,
                    &servername,sizeof(struct sockaddr));
          if (rv==(-1)) {
               logit("Problem with sendto().");
               if (++count>SEND_RETRIES) {
                    netShutdown();
                    return(RV_nBADSEND);
               }
          }
     } while (rv!=PACKET_SIZE);
     netShutdown();
     return(RV_nOK);
}


#ifdef __STDC__
int AlarmHandler(int sig)
#else
int AlarmHandler(sig)
int sig;
#endif
{
     longjmp(jumpTimeout,1);
     return(0);
}


#ifdef __STDC__
int packetReceive(USEREC *msg)
#else
int packetReceive(msg)
USEREC *msg;
#endif
{
     u_char chk;
     int i;
     PACKET in;
     char buf[132];

     if (netSetup()) return(RV_nBADRECV);
     do {
          signal(SIGALRM,AlarmHandler);
          if (setjmp(jumpTimeout)!=0) return(RV_nTIMEOUT);
          alarm(TIMEOUT);
          i=sizeof(struct sockaddr);
          if (recvfrom(sock,in,PACKET_SIZE,0,
                       &servername,&i)==(-1)) {
               logit("Problem with recvfrom().");
               netShutdown();
               return(RV_nBADRECV);
          }
          alarm(0);
          signal(SIGALRM,SIG_DFL);
          bcopy(in,msg,sizeof(PACKET));
#ifdef DEBUG
          logit("Client received:");
          sprintf(buf,"Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x Users=%04x Console=%04x /tmp=%08x Mach=%02x Model=%02x Chksum=%02x",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3),ntohs(msg->users),
                 ntohs(msg->console),ntohl(msg->tmp),msg->machine,msg->model,in[0]);
          logit(buf);
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
     } while (chk!=in[0]);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     netShutdown();
     return(RV_nOK);
}


#ifdef __STDC__
int netSetup(void)
#else
int netSetup()
#endif
{
     struct hostent *hp;
     struct servent *servptr;
     int portno;

     sock=socket(AF_INET,SOCK_DGRAM,0);
     if (sock<0) {
          logit("Problem acquiring main socket");
          return(1);
     }
     clientname.sin_family=AF_INET;
     clientname.sin_addr.s_addr=INADDR_ANY;
     clientname.sin_port=0;
     if (bind(sock,&clientname,sizeof(clientname))) {
          logit("Unable to bind main socket");
          close(sock);
          return(1);
     }
     servptr=getservbyname(NETUSE_SERVICE,NULL);
     if (servptr==NULL) htons(portno=NETUSE_PORT);
     else portno=htons(servptr->s_port);
     hp=gethostbyname(NETUSE_SERVER);
     if (hp==NULL) {
          logit("Unable to get server address");
          close(sock);
          return(1);
     }
     bcopy(hp->h_addr,&servername.sin_addr,hp->h_length);
     servername.sin_family=AF_INET;
     servername.sin_port=htons(portno);
     return(0);
}


#ifdef __STDC__
void netShutdown(void)
#else
void netShutdown()
#endif
{
     close(sock);
}


#ifdef __STDC__
int report(float load1,float load2,float load3,int users,int console,long tmp)
#else
int report(load1,load2,load3,users,console,tmp)
float load1,load2,load3;
int users,console;
long tmp;
#endif
{
     USEREC msg;

     msg.opcode=OP_STATUS;
     msg.load1=(u_short)(load1*100);
     msg.load2=(u_short)(load2*100);
     msg.load3=(u_short)(load3*100);
     msg.users=(u_short)users;
     msg.console=(u_short)console;
     msg.tmp=(u_long)tmp;
     msg.machine=(u_char)MACHINETYPE;
     msg.model=(u_char)modelMine;
     msg.retcode=RV_OK;
     packetSend(&msg);
#ifdef ACKNOWLEDGE
     if (err=packetReceive(&msg)) {
          switch (err) {
               case RV_nTIMEOUT:
                    return(RV_TIMEOUT);
          }
     }
#endif
     return(0);
}


#ifdef __STDC__
void inputClean(void)
#else
void inputClean()
#endif
{
     fd_set read_template;                    /* Descriptors to be read from */
     struct timeval tvTimeout;
     char inbuf[1024],buf[132];
     int more,rv,size;
     struct hostent *hp;
     struct servent *servptr;
     int i;

     i=0;
     do {
#ifdef DEBUG
          sprintf(buf,"Checking for waiting input...",rv);
          logit(buf);
#endif
          FD_ZERO(&read_template);
          FD_SET(0,&read_template);
          tvTimeout.tv_sec=0;
          tvTimeout.tv_usec=0;
          rv=select(FD_SETSIZE,&read_template,(fd_set *)0,(fd_set *)0,
                    &tvTimeout);
          if (rv<0) {
#ifdef DEBUG
                sprintf(buf,"select() returned error %d.",rv);
                logit(buf);
#endif
                return;
          }
          if (more=FD_ISSET(0,&read_template)) {
#ifdef DEBUG
               sprintf(buf,"Found waiting input.  Reading it.",rv);
               logit(buf);
#endif
               size=0;
               if ((rv=recvfrom(0,inbuf,sizeof(inbuf),0,NULL,&size))<0) {
                    if (errno==ENOTSOCK) {
                         logit("Input is not a socket -- ignoring.");
                         more=0;
                    }
                    if (errno==EBADF) {
                         logit("Bad descriptor on input -- ignoring.");
                         more=0;
                    }
                    if (i>1000) {
                         logit("Too many errors.  Ignoring input.");
                         more=0;
                    }
                    sprintf(buf,"recvfrom() returned error %d.",errno);
                    logit(buf);
                    ++i;
                    if (i>=100) {
                         logit("Too many errors.  Ignoring input.");
                         more=0;
                    }
               }
#ifdef DEBUG
               else {
                    sprintf(buf,"Read %d bytes of input.\n",rv);
                    logit(buf);
               }
#endif
          }
          else {
#ifdef DEBUG
               sprintf(buf,"No available input.",rv);
               logit(buf);
#endif
          }
     } while (more);
/*
#ifdef DEBUG
     logit("Before close().");
#endif
     close(0);
#ifdef DEBUG
     logit("After close().");
#endif
*/
}


#ifdef __STDC__
void StopEverything(void)
#else
void StopEverything()
#endif
{
     char hostname[132],buf[132];
     int rv;

     gethostname(hostname,sizeof(hostname));
     if (rv=netuseDelName(hostname)) {
          sprintf(buf,"netuseDelName() returned %d.",rv);
          logit(buf);
     }
     logit("Not gonna run.");
     exit(0);
}


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     int time,period,mode,i,awakenings;
     double GetLoadPoint();
     long getdisk();
     int getusers();
     u_char modelDetect();
     double l1,l2,l3;
     int users,console;
     int fd;
     long tmp;
     char temp[132],buf[512];

     inputClean();
#ifdef hpux
     if (setsid()<0) logit("setsid() returned an error.");
#else
     if ((fd=open("/dev/tty",O_RDWR,0))!=(-1)) {   /* Detach controlling tty */
          ioctl(fd,TIOCNOTTY,0);
          close(fd);
     }
#endif
     /* if (fork()) exit(0); */
     mode=0;
     time=DEFAULT_PERIOD;
     period=REPORT_CYCLE;
     for (i=1; argc>i; i++)
          if ((argv[i][0]=='-') || (argv[i][0]=='/')) {
               strcpy(temp,argv[i]+1);
               stoupper(temp);
               if (!strcmp(temp,"ONCE")) mode=1;
               if (!strcmp(temp,"FORCE")) mode=2;
               if (!strcmp(temp,"WAKEUP")) {
                    if (argc>(++i)) time=atoi(argv[i]);
                    else logit("Missing value for -wakeup option.");
               }
               if (!strcmp(temp,"REPORT")) {
                    if (argc>(++i)) period=atoi(argv[i]);
                    else logit("Missing value for -report option.");
               }
               if (!strcmp(temp,"INETD")) {
                    while (read(0,buf,512))           /* Empty socket buffer */
                         ;
                    close(0); close(1);
               }
          }
#ifdef USE_CAENLAB
     if (!netusedShouldRun() && !mode) StopEverything();
#endif
     InitGetLoad();
     modelMine=modelDetect();
     awakenings=REPORT_CYCLE;
     do {
          GetLoadPoint(&l1,&l2,&l3);
          getusers(&users,&console);
          tmp=getdisk();
          if (awakenings==REPORT_CYCLE) {
               report((float)l1,(float)l2,(float)l3,users,console,tmp);
               awakenings=1;
          }
          else ++awakenings;
          if (mode!=1) sleep(time);
     } while (mode!=1);
}
