/******************************************************************************
 WAKEUP.C - Wake up a host's netused by sending a datagram to the netused port

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <signal.h>
#include <setjmp.h>
#include <pwd.h>
#include <errno.h>
#if (defined(_AIX) || defined(apollo))
#include <sys/ioctl.h>
#else
#include <sys/termios.h>
#endif

#include "../lib/protocol.h"
#include "../lib/netuse.h"
#include "../lib/config.h"
#include "../lib/variable.h"


extern long lTime;
extern VARTABLE variables;

int sock;
struct sockaddr_in myname;
jmp_buf jumpTimeout;
long lastCheck=0,lastSave=0,netuseStartTime;


#ifdef __STDC__
int packetSend(USEREC *msg,struct sockaddr_in *dest)
#else
int packetSend(msg,dest)
USEREC *msg;
struct sockaddr_in *dest;
#endif
{
     u_char chk;
     int rv,count,i;
     PACKET out;

     msg->load1=htons(msg->load1);
     msg->load2=htons(msg->load2);
     msg->load3=htons(msg->load3);
     msg->users=htons(msg->users);
     msg->console=htons(msg->console);
     msg->tmp=htonl(msg->tmp);
     msg->uid=htons(msg->uid);
     bcopy(msg,out,sizeof(PACKET));
     for (chk=0,i=1; i<PACKET_SIZE; i++)
          chk=(chk+out[i]) % 0xFF;
     out[0]=chk;
#ifdef DEBUG
     printf("Server sending:\n");
     printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
            msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
            ntohs(msg->load2),ntohs(msg->load3));
     printf("Users=%04x Console=%04x Tmp=%08x Mach=%2x Model=%02x Chksum=%02x\n",
            ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
            msg->machine,msg->model,out[0]);
     printf("Data:");
     for (i=0; i<PACKET_SIZE; i++) {
          if (i && !(i % 16)) printf("\n     ");
          printf(" %02x",out[i]);
     }
     printf("\n");
#endif
     count=0;
     do {
          rv=sendto(sock,out,PACKET_SIZE,0,dest,sizeof(struct sockaddr_in));
          if (rv==(-1)) {
               perror("sendto");
               if (++count>SEND_RETRIES) return(RV_nBADSEND);
          }
     } while (rv!=PACKET_SIZE);
     return(RV_nOK);
}


#ifdef __STDC__
int AlarmHandler(int sig)
#else
int AlarmHandler(sig)
int sig;
#endif
{
     longjmp(jumpTimeout,1);
     return(0);
}


#ifdef __STDC__
int packetReceiveT(USEREC *msg,struct sockaddr_in *src)
#else
int packetReceiveT(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     u_char chk;
     int i;
     PACKET in;

     do {
          signal(SIGALRM,AlarmHandler);
          if (setjmp(jumpTimeout)!=0) return(RV_nTIMEOUT);
          alarm(TIMEOUT);
          i=sizeof(struct sockaddr_in);
          if (recvfrom(sock,in,PACKET_SIZE,0,src,&i)==(-1)) {
               perror("recvfrom");
               return(RV_nBADRECV);
          }
          alarm(0);
          signal(SIGALRM,SIG_DFL);
          bcopy(in,msg,sizeof(PACKET));
#ifdef DEBUG
          printf("Server received:\n");
          printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3));
          printf("Users=%04x Console=%04x Tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
                 ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
                 msg->machine,msg->model,in[0]);
          printf("Data:");
          for (i=0; i<PACKET_SIZE; i++) {
               if (i && !(i % 16)) printf("\n     ");
               printf(" %02x",in[i]);
          }
          printf("\n");
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
     } while (chk!=msg->chksum);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     return(RV_nOK);
}


#ifdef __STDC__
int packetReceive(USEREC *msg,struct sockaddr_in *src)
#else
int packetReceive(msg,src)
USEREC *msg;
struct sockaddr_in *src;
#endif
{
     u_char chk;
     int i;
     PACKET in;

     do {
          i=sizeof(struct sockaddr_in);
          if (recvfrom(sock,in,PACKET_SIZE,0,src,&i)==(-1)) {
               perror("recvfrom");
               return(RV_nBADRECV);
          }
          bcopy(in,msg,sizeof(PACKET));
#ifdef DEBUG
          printf("Server received:\n");
          printf("Opcode=%02x Retcode=%02x Ack=%02x Load1=%04x Load2=%04x Load3=%04x\n",
                 msg->opcode,msg->retcode,msg->ack,ntohs(msg->load1),
                 ntohs(msg->load2),ntohs(msg->load3));
          printf("Users=%04x Console=%04x Tmp=%08x Mach=%02x Model=%02x Chksum=%02x\n",
                 ntohs(msg->users),ntohs(msg->console),ntohl(msg->tmp),
                 msg->machine,msg->model,in[0]);
          printf("PACKET_SIZE=%d\n",PACKET_SIZE);
          printf("Data:");
          for (i=0; i<PACKET_SIZE; i++) {
               if (i && !(i % 16)) printf("\n     ");
               printf(" %02x",in[i]);
          }
          printf("\n");
#endif
          for (chk=0,i=1; i<PACKET_SIZE; i++)
               chk=(chk+in[i]) % 0xFF;
#ifdef DEBUG
          if (chk!=msg->chksum)
               printf("Bad checksum! %2x should be %2x\n",msg->chksum,chk);
#endif
     } while (chk!=msg->chksum);
     msg->load1=ntohs(msg->load1);
     msg->load2=ntohs(msg->load2);
     msg->load3=ntohs(msg->load3);
     msg->users=ntohs(msg->users);
     msg->console=ntohs(msg->console);
     msg->tmp=ntohl(msg->tmp);
     msg->uid=ntohs(msg->uid);
     return(RV_nOK);
}


#ifdef __STDC__
void netSetup(void)
#else
void netSetup()
#endif
{
     struct servent *servptr;
     int portno;

     sock=socket(AF_INET,SOCK_DGRAM,0);
     if (sock<0) {
          perror("Acquiring a socket");
          exit(1);
     }
     myname.sin_family=AF_INET;
     myname.sin_addr.s_addr=INADDR_ANY;
     myname.sin_port=0;
     if (bind(sock,&myname,sizeof(myname))) {
          perror("Binding socket");
          exit(1);
     }
}


#ifdef __STDC__
void netShutdown(void)
#else
void netShutdown()
#endif
{
     close(sock);
}


/*****************************************************************************
 void logit(msg)

 This function logs a message in the (previously opened) logfile.  Note that
 fflush() is used to ensure that the message gets logged to disk.
*****************************************************************************/

#ifdef __STDC__
void logit(char *msg)
#else
void logit(msg)
char *msg;
#endif
{
     char atime[64];
     struct timeval tv;
     struct timezone tz;
     int retries;
     FILE *logf;

     gettimeofday(&tv,&tz);
     strcpy(atime,asctime(localtime(&(tv.tv_sec))));
     atime[strlen(atime)-1]='\0';
     for (retries=0; (retries<5) && ((logf=fopen(NETUSELOG,"a"))==NULL); ++retries)
          sleep(5);
     if (retries==5) {
          fprintf(stderr,"Unable to open log file %s for update.\n",NETUSELOG);
          return;
     }
     fprintf(logf,"%s -- %s\n",atime,msg);
     fclose(logf);
     chmod(NETUSELOG,0644);
}


#ifdef __STDC__
void hostWakeup(char *host)
#else
void hostWakeup(host)
char *host;
#endif
{
     struct servent *servptr;
     struct sockaddr_in dest;
     u_long ip;
     char buf[128];

     servptr=getservbyname(NETUSED_SERVICE,NULL);
     if (servptr==NULL) dest.sin_port=NETUSED_PORT;
     else dest.sin_port=servptr->s_port;
     dest.sin_family=AF_INET;
     if (!(ip=netGetAddress(host))) {
          fprintf(stderr,"Unable to get IP address for %s.\n",host);
          return;
     }
     dest.sin_addr.s_addr=ip;
     printf("Yo!  (Waking up %s)...\n",host);
     if (sendto(sock,"Wakeup!",9,0,&dest,sizeof(dest))==(-1)) {
          sprintf(buf,"Error %d from sendto() in WakeTheDead().\n",errno);
          logit(buf);
     }
     close(socket);
}


#ifdef __STDC__
main(int argc,char *argv[])
#else
main(argc,argv)
int argc;
char *argv[];
#endif
{
     int i;
     char buffer[1024];

     netSetup();
     for (i=1; i<argc; i++) {
          strcpy(buffer,argv[i]);
          stoupper(buffer);
          if (buffer[0]=='-') {
               if (!strcmp(buffer,"-HELP")) {
                    printf("\nUsage: %s [options] host1 ...\n\n",argv[0]);
                    printf("Options: -help            Display usage info\n\n");
               }
          }
          else hostWakeup(buffer);
     }
     netShutdown();
}
