/******************************************************************************
 CONFIG.H - NETUSE customization header file

 This header file defines site-dependent customizations.

 Lee Liming, Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#ifndef _CONFIG_INCLUDED_
#define _CONFIG_INCLUDED_


/*  The NETUSE_SERVER macro should expand to the IP hostname of the machine  */
/*  on which the netuse daemon for your site runs.                           */

#define NETUSE_SERVER   "myhost.some.where"

/*  At CAEN, we have some unusual ways of determining whether or not the     */
/*  netused daemon should run on a given host.  The existence of USE_CAENLAB */
/*  says that netused will use those CAEN-specific heuristics.  Other sites  */
/*  should not use this.                                                     */

/* #define USE_CAENLAB */

/*  The name of the file in which the netused daemon stores problem reports. */

#define NETUSEDLOG      "/tmp/netused.log"

/*  The name of the file in which the netuse daemon stores log messages.     */

#define NETUSELOG       "/usr/adm/netuse-req.log"

/*  The name of the file in which the netuse daemon stores host information. */

#define NETUSEHOSTFILE  "/usr/adm/netuse.hosts"

/* The types of log messages that routinely get logged by the netuse daemon. */
/* E.g., to log all NETUSE requests, uncomment LOGREQUESTS.  To log all      */
/* wakeup calls sent by netuse, uncomment LOGWAKEUPS.  These log files can   */
/* get pretty big, though...                                                 */

/* #define LOGWAKEUPS */
#define LOGREQUESTS

/* The default port numbers and service names used by netuse and netused.    */

#define NETUSED_PORT    1689           /* Used if /etc/services doesn't know */
#define NETUSE_PORT     1690           /* Used if /etc/services doesn't know */
#define NETUSED_SERVICE "netused"    /* Inet service name for client daemons */
#define NETUSE_SERVICE  "netuse"      /* Inet service name for NETUSE server */

/* The number of hosts to be listed by netuse if -all is not specified.      */

#define MAXLSTLEN         10

/* Number of minutes without a report from a node before netuse considers    */
/* the node "down".                                                          */

#define DOWNTIME          15

/* Number of minutes without a report from a node before netuse deletes the  */
/* node from its database.                                                   */ 

#define GIVEUPTIME     10080

/* Default period (in seconds) between reports by netused.                   */
/* Note that AIX load averaging requires that netused wake up around once    */
/* every 15 seconds, but it still reports once every 8 wakeups (2 min.)      */

#ifdef _AIX
#define DEFAULT_PERIOD    15           /* Period between wakeups for netused */
#define REPORT_CYCLE       8       /* Number of wakeups in each report cycle */
#else
#define DEFAULT_PERIOD   120
#define REPORT_CYCLE       1
#endif


#endif /* _CONFIG_INCLUDED_ */
