/******************************************************************************
 GETHOST.C - Client routines for finding hosts of various types

 The routines in this file are all related to querying the NETUSE server to
 find hosts which match certain criteria.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#ifdef IBMTCPIP
#include <types.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "protocol.h"
#include "network.h"
#include "netuse.h"


typedef struct _models {
             char   name[15];
             u_char num;
        } MODELS;

MODELS models[]={{"3100",MODEL_DS3100},{"5000/200",MODEL_DS5000200},
                 {"5000/120",MODEL_DS5000120},{"4/?",MODEL_SUN4},
                 {"4/65",MODEL_SUN4_65},{"4/50",MODEL_SUN4_50},
                 {"6000/320",MODEL_RS320},{"6000/520",MODEL_RS520},
                 {"6000/530",MODEL_RS530},{"6000/540",MODEL_RS540},
                 {"6000/730",MODEL_RS730},{"6000/930",MODEL_RS930},
                 {"3000",MODEL_AP3000},{"3500",MODEL_AP3500},
                 {"4000",MODEL_AP4000},{"4500",MODEL_AP4500},
                 {"5500",MODEL_AP5500},{"10010",MODEL_AP10010},
                 {"10020",MODEL_AP10020},{"425E",MODEL_HP425E},
                 {"425T",MODEL_HP425T},{"9000/720",MODEL_HP9000720},
                 {"3/?",MODEL_SUN3},{"2500",MODEL_AP2500},
                 {"5000/133",MODEL_DS5000133},{"9000/705",MODEL_HP9000705},
                 {"9000/710",MODEL_HP9000710},{"9000/730",MODEL_HP9000730},
                 {"9000/750",MODEL_HP9000750},{"6000/220",MODEL_RS220},
                 {"6000/320H",MODEL_RS320H},{"6000/340",MODEL_RS340},
                 {"6000/350",MODEL_RS350},{"6000/530H",MODEL_RS530H},
                 {"6000/550",MODEL_RS550},{"6000/560",MODEL_RS560},
                 {"6000/950",MODEL_RS950},
                 {"?",0}};


/******************************************************************************
 u_long netGetAddress(char *buf)

 This function coverts an internet hostname into an internet address (numeric
 format).  If no address can be found for the name, the hostname is checked to
 see if it is actually a numeric address in text form (e.g., "141.212.66.69").
 If it is, it is converted verbatim into numeric format.  If not, the return
 value is 0.  If multiple addresses are found for the host, the first one is
 returned.  The resulting IP address is given in network format.
******************************************************************************/

#ifdef __STDC__
u_long netGetAddress(char *buf)
#else
u_long netGetAddress(buf)
char *buf;
#endif
{
     struct hostent *hp;
     u_long addrbuf;
     u_char ipaddr[4];
     int i;
     char temp[64],*p,*q,**cpp;

#if (defined(MSDOS) && defined(NOVELL))
     cpp=(&buf);
     if ((addrbuf=rhost(cpp))==(-1)) {
          perror("Getting server address");
          addrbuf=(u_long)0;
     }
#else
     hp=gethostbyname(buf);
     if (hp==NULL) {
          p=buf;
          for (i=0; i<4; i++) {
               q=temp;
               while ((*p!='\0') && (*p!='.')) *(q++)=(*(p++));
               *q='\0';
               if (*p) ++p;
               ipaddr[i]=(u_char)atoi(temp);
          }
          bcopy(ipaddr,&addrbuf,sizeof(addrbuf));
     }
     else {
          if (hp->h_addr_list[0]!=NULL)
               bcopy(hp->h_addr_list[0],&addrbuf,sizeof(addrbuf));
          else addrbuf=0;
     }
#endif
     return(addrbuf);
}


/******************************************************************************
 char *netGetHostname(u_long inaddr,char *buf)

 This function is the inverse of netGetAddress().  The IP address in network
 format stored in inaddr is converted to an internet hostname.  The resulting
 name is stored in the string pointed to by buf.  If no hostname can be found
 for the address, an ASCII representation of the IP address is returned (e.g.,
 "141.212.66.69").  A pointer to the name buffer (buf) is the return value.
******************************************************************************/

#ifdef __STDC__
char *netGetHostname(u_long inaddr,char *buf)
#else
char *netGetHostname(inaddr,buf)
u_long inaddr;
char *buf;
#endif
{
     struct hostent *hp;
     u_char ipbytes[4];
     char *cp;

#if (defined(MSDOS) && defined(NOVELL))
     if ((cp=raddr(inaddr))==(char *)(-1)) {
          bcopy((char *)&inaddr,ipbytes,sizeof(ipbytes));
          sprintf(buf,"%d.%d.%d.%d",ipbytes[0],ipbytes[1],ipbytes[2],ipbytes[3]);
     }
     else strcpy(buf,cp);
#else
     hp=gethostbyaddr(&inaddr,sizeof(u_long),AF_INET);
     if (hp==NULL) {
          bcopy(&inaddr,ipbytes,sizeof(ipbytes));
          sprintf(buf,"%d.%d.%d.%d",ipbytes[0],ipbytes[1],ipbytes[2],ipbytes[3]);
     }
     else strcpy(buf,hp->h_name);
#endif
     return(buf);
}


/******************************************************************************
 char *netGetMachType(u_char mtype,char *mtname)

 This function returns a text string representing the vendor type given by
 the mtype parameter, according to the MACH_xxx constants in netuse.h.  The
 resulting string is placed in the buffer pointed to by mtname, and a pointer
 to mtname is returned.  If mtype is not a valid machine type, the string "*"
 is returned.
******************************************************************************/

#ifdef __STDC__
char *netGetMachType(u_char mtype,char *mtname)
#else
char *netGetMachType(mtype,mtname)
u_char mtype;
char *mtname;
#endif
{
     switch (mtype) {
          case MACH_DEC:
               strcpy(mtname,"DEC");
               break;
          case MACH_SUN:
               strcpy(mtname,"Sun");
               break;
          case MACH_IBM_RS6000:
               strcpy(mtname,"IBM");
               break;
          case MACH_APOLLO:
               strcpy(mtname,"Apo");
               break;
          case MACH_HP:
               strcpy(mtname,"HP ");
               break;
          default:
               strcpy(mtname,"*");
               break;
     }
     return(mtname);
}


/******************************************************************************
 This is the inverse of the netGetMachType() function.  It returns the numeric
 representation of the vendor name pointed to by mtname, according to the
 values given in netuse.h.  If the string is not a valid vendor type, the
 return value is 0.  The mtname string is treated case-insensitively.
******************************************************************************/

#ifdef __STDC__
u_char netStrToMach(char *mtname)
#else
u_char netStrToMach(mtname)
char *mtname;
#endif
{
     char buf[128];

     if (mtname==NULL) return(0);
     strcpy(buf,mtname);
     stoupper(buf);
     if (!strcmp(buf,"SUN")) return(MACH_SUN);
     if (!strcmp(buf,"DEC")) return(MACH_DEC);
     if (!strcmp(buf,"IBM")) return(MACH_IBM_RS6000);
     if (!strncmp(buf,"APO",3)) return(MACH_APOLLO);
     if (!strncmp(buf,"HP",2)) return(MACH_HP);
     return(0);
}


/******************************************************************************
 char *netGetModelType(u_char mtype,char *mtname)

 This function returns a text string representing the model type given by
 the mtype parameter, according to the MODEL_xxx constants in netuse.h.  The
 resulting string is placed in the buffer pointed to by mtname, and a pointer
 to mtname is returned.  If mtype is not a valid machine type, the string "*"
 is returned.
******************************************************************************/

#ifdef __STDC__
char *netGetModelType(u_char mtype,char *mtname)
#else
char *netGetModelType(mtype,mtname)
u_char mtype;
char *mtname;
#endif
{
     MODELS *mptr;

     if (mtname==NULL) return(NULL);
     for (mptr=models; mptr->num && (mptr->num!=mtype); mptr++)
          ;
     strcpy(mtname,mptr->name);
     return(mtname);
}


/******************************************************************************
 This is the inverse of the netGetModelType() function.  It returns the numeric
 representation of the model name pointed to by mtname, according to the
 values given in netuse.h.  If the string is not a valid vendor type, the
 return value is 0.  The mtname string is treated case-insensitively.
******************************************************************************/

#ifdef __STDC__
u_char netStrToModel(char *mtname)
#else
u_char netStrToModel(mtname)
char *mtname;
#endif
{
     char buf[128];
     MODELS *mptr;

     if (mtname==NULL) return(0);
     strcpy(buf,mtname);
     stoupper(buf);
     for (mptr=models; mptr->num && strcmp(buf,mptr->name); mptr++)
          ;
     return(mptr->num);
}


/******************************************************************************
 char *netuseFind(name,l1,l2,l3,users,console,tmp,machine,model)

 This function queries the NETUSE server for a machine which matches the
 parameters given.  The name of a matching machine is placed in the buffer
 pointed to by name, and a pointer to the buffer is returned.  If no host
 could be found, the name string is empty ("").  Network or server errors
 simply cause the return value to be the empty string.

 The following values give the "wildcard" values for each parameter.  If the
 parameter is set to the wildcard value, that parameter will not be used in
 selecting a host.

    Parameter   Description                          Wildcard
    ----------- ------------------------------------ --------
    l1,l2,l3    Max load averages                      100.0
    users       Max number of interactive sessions     100
    console     Max console sessions                   100
    tmp         Min /tmp free space (bytes)              0
    machine     Machine (vendor) type                    0
    model       Machine (model) type                     0
******************************************************************************/

#ifdef __STDC__
char *netuseFind(char *name,double l1,double l2,double l3,int users,int console,
                 int tmp,u_char machine,u_char model)
#else
char *netuseFind(name,l1,l2,l3,users,console,tmp,machine,model)
char *name;
double l1,l2,l3;
int users,console,tmp;
u_char machine,model;
#endif
{
     int rv,retry;
     USEREC msg;

     msg.opcode=OP_QUERY;
     msg.load1=(u_short)(l1*100);
     msg.load2=(u_short)(l2*100);
     msg.load3=(u_short)(l3*100);
     msg.users=(u_short)users;
     msg.console=(u_short)console;
     msg.tmp=(u_long)tmp;
     msg.machine=machine;
     msg.model=model;
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=(u_short)getuid();
#endif
     if (rv=packetSend(&msg)) {
          strcpy(name,"");
          return(name);
     }
     retry=0;
     while (rv=packetReceive(&msg)) {
          switch (rv) {
               case RV_nBADRECV:
               case RV_nTIMEOUT:
                    if ((++retry)>REQ_RETRIES) {
                         strcpy(name,"");
                         return(name);
                    }
                    msg.opcode=OP_QUERY;
                    msg.load1=(u_short)(l1*100);
                    msg.load2=(u_short)(l2*100);
                    msg.load3=(u_short)(l3*100);
                    msg.users=(u_short)users;
                    msg.console=(u_short)console;
                    msg.tmp=(u_long)tmp;
                    msg.machine=machine;
                    msg.model=model;
                    packetSend(&msg);
                    break;
          }
     }
     if (msg.retcode==RV_OK) {
          msg.ipaddr=htonl(msg.ipaddr);   /* Put back into network order */
          return(netGetHostname(msg.ipaddr,name));
     }
     else {
          strcpy(name,"");
          return(name);
     }
}
