/******************************************************************************
 GETLIST.C - NETUSE Client List-receiving Routines

 The functions defined in this module are used to receive host lists from the
 NETUSE server.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#ifdef IBMTCPIP
#include <types.h>
#endif
#ifndef MSDOS
#include <sys/time.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "network.h"
#include "protocol.h"
#include "netuse.h"


extern long netuseTimeout;                                 /* From network.c */

char buf1[255];
int buf1bytes=0,bufptr=0;


/******************************************************************************
 This function queries the NETUSE server for a host list of a type specified
 by the machine (vendor type) and mode (listing mode) parameters.  The list is
 printed directly on stdout (Ok, ok...  So write you're own!).

 The function uses a UDP packet for the initial request, and includes the port
 number of a TCP socket which it wants to receive the list at.  The server
 will send the list in text format to this TCP socket.  If the server never
 responds, this function will wait until the accept() fails before returning
 (i.e., there's no timeout on this one).

 Valid modes and machine and model types are given in netuse.h.
******************************************************************************/

#ifdef __STDC__
int netuseGetList(u_char machine,u_char model,int mode)
#else
int netuseGetList(machine,model,mode)
u_char machine,model;
int mode;
#endif
{
     USEREC msg;
     int tcpsock,readsock,i,rv;
     struct sockaddr_in newaddr;
     FILE *inf;
     char buf[132],*ch,eol[3];
#ifdef IBMTCPIP
     int fdset[2];
     long tv;
#else
     struct fd_set fdset;
     struct timeval tv;
#endif

     tcpsock=socket(AF_INET,SOCK_STREAM,0);
     if (tcpsock<0) {
          perror("socket");
          netShutdown();
          return(1);
     }
     newaddr.sin_family=AF_INET;
     newaddr.sin_addr.s_addr=INADDR_ANY;
     newaddr.sin_port=0;
     if (bind(tcpsock,(struct sockaddr *)&newaddr,sizeof(newaddr))) {
          perror("bind");
          return(1);
     }
     rv=listen(tcpsock,2);
#ifdef DEBUG
     if (rv) printf("listen() returned %d in netuseGetList().\n",rv);
#endif
     if (mode & MODE_MODELS) {
          switch (mode & MODE_MASK) {
               case MODE_DISP:
                    msg.opcode=OP_DISPMLIST;
                    break;
               case MODE_ALL:
                    msg.opcode=OP_GETMLIST;
                    break;
               case MODE_DOWN:
                    msg.opcode=OP_GETMDOWN;
                    break;
          }
     }
     else {
          switch (mode & MODE_MASK) {
               case MODE_DISP:
                    msg.opcode=OP_DISPLIST;
                    break;
               case MODE_ALL:
                    msg.opcode=OP_GETLIST;
                    break;
               case MODE_DOWN:
                    msg.opcode=OP_GETDOWN;
                    break;
          }
     }
     msg.machine=machine;
     msg.model=model;
     i=sizeof(newaddr);
     if (rv=getsockname(tcpsock,(struct sockaddr *)&newaddr,&i)) {
#ifdef DEBUG
          printf("getsockname() returned %d in netuseGetList().\n",rv);
#endif
          return(-1);
     }
#ifdef DEBUG
     printf("Using port number %04x in netuseGetList().\n",newaddr.sin_port);
#endif
     msg.portno=ntohs(newaddr.sin_port);
#ifdef DEBUG
     printf("Sending port number %04x in netuseGetList().\n",msg.portno);
#endif
#ifdef MSDOS
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if (rv=packetSend(&msg)) {
#ifdef DEBUG
          printf("packetSend() returned %d in netuseGetList().\n",rv);
#endif
     }
#ifdef IBMTCPIP
     fdset[0]=tcpsock;
     tv=netuseTimeout*(REQ_RETRIES+1)*1000;
#ifdef DEBUG
     printf("About to call IBM's lobotomized select() in netuseGetList().\n");
     fflush(stdout);
#endif
     rv=select(fdset,1,0,0,tv);
#else /* ! IBMTCPIP */
     FD_ZERO(&fdset);
     FD_SET(tcpsock,&fdset);
     tv.tv_sec=netuseTimeout*(REQ_RETRIES+1);
     tv.tv_usec=0;
#ifdef DEBUG
     printf("About to call select() in netuseGetList().\n");
     fflush(stdout);
#endif
     rv=select(FD_SETSIZE,&fdset,NULL,NULL,&tv);
#endif /* !IBMTCPIP */
#ifdef DEBUG
     printf("Back from select() in netuseGetList().\n");
     fflush(stdout);
#endif
     if (rv<0) {
          fprintf(stderr,"netuseGetList(): Error %d from select().\n",rv);
#if (defined(MSDOS) || defined(OS2))
          soclose(tcpsock);
#else
          close(tcpsock);
#endif
          return(RV_nBADRECV);
     }
#ifdef IBMTCPIP
     if (fdset[0]!=(-1)) {
#else
     if (FD_ISSET(tcpsock,&fdset)) {
#endif
#ifdef DEBUG
          printf("About to call accept() in netuseGetList()...\n");
          fflush(stdout);
#endif
          if ((readsock=accept(tcpsock,(struct sockaddr *)&newaddr,&i))<0) {
               perror("accept");
#if (defined(MSDOS) || defined(IBMTCPIP))
               soclose(tcpsock);
#else
               close(tcpsock);
#endif
               return(1);
          }
#ifdef DEBUG
          printf("Successful accept() in netuseGetList().\n");
          fflush(stdout);
#endif
#if (defined(MSDOS) || defined(IBMTCPIP))
          soclose(tcpsock);
#else
          close(tcpsock);
#endif
     }
     else {
#if (defined(MSDOS) || defined(IBMTCPIP))
          soclose(tcpsock);
#else
          close(tcpsock);
#endif
#ifdef DEBUG
          printf("select() failed to get a connection in netuseGetList().\n");
          fflush(stdout);
#endif
          return(RV_nTIMEOUT);
     }
     if (mode & MODE_NET) strcpy(eol,"\r\n");
     else strcpy(eol,"\n");
     printf("%s",eol);
     while (!netuseTCPReceiveMessage(readsock,buf)) {
          for (ch=buf; *ch!='\0'; ch++)
               if ((*ch=='\n') || (*ch=='\r')) strcpy(ch,ch+1);
          printf("%s%s",buf,eol);
     }
#if (defined(MSDOS) || defined(IBMTCPIP))
     soclose(readsock);
#else
     close(readsock);
#endif
     return(0);
}


#ifdef __STDC__
int netuseTCPReceiveMessage(int sock,char *msg)
#else
int netuseTCPReceiveMessage(sock,msg)
int sock;
char *msg;
#endif
{
     int bytesRead,done,rv;
     char *bp;
#ifdef IBMTCPIP
     int fdset[2];
     long tv;
#else
     struct fd_set fdset;
     struct timeval tv;
#endif

#ifdef DEBUG
     printf("\nInside netuseTCPReceiveMessage().\n");
#endif
     done=0; bp=msg;
     do {
          for ( ; !done && (bufptr<buf1bytes); bufptr++) {
               if ((*(bp++)=buf1[bufptr])=='\n') {
                    *bp='\0';
                    done=1;
               }
          }
          if (!done) {
#ifdef IBMTCPIP
               fdset[0]=sock;
               tv=netuseTimeout*(REQ_RETRIES+1)*1000;
#ifdef DEBUG
               printf("About to call IBM's lobotomized select() in netuseTCPReceiveMessage().\n");
               fflush(stdout);
#endif
               rv=select(fdset,1,0,0,tv);
#else /* ! IBMTCPIP */
               FD_ZERO(&fdset);
               FD_SET(sock,&fdset);
               tv.tv_sec=netuseTimeout*(REQ_RETRIES+1);
               tv.tv_usec=0;
#ifdef DEBUG
               printf("About to call select() in netuseTCPReceiveMessage().\n");
               fflush(stdout);
#endif
               rv=select(FD_SETSIZE,&fdset,NULL,NULL,&tv);
#endif /* ! IBMTCPIP */
#ifdef DEBUG
               printf("select() in netuseTCPReceiveMessage() returns %d.\n",rv);
               fflush(stdout);
#endif
               if (rv==(-1)) {
                    fprintf(stderr,"select: Error in netuseTCPReceiveMessage().\n",rv);
                    return(RV_nBADRECV);
               }
#ifdef IBMTCPIP
               if (fdset[0]!=(-1)) {
#else
               if (FD_ISSET(sock,&fdset)) {
#endif
#ifdef DEBUG
                    printf("Calling soread() in netuseTCPReceiveMessage()...\n",rv);
#endif
#ifdef NOVELL
                    if ((buf1bytes=soread(sock,buf1,sizeof(buf1)))<=0) {
#else
#ifdef IBMTCPIP
                    if ((buf1bytes=recv(sock,buf1,sizeof(buf1),0))<=0) {                    
#else
                    if ((buf1bytes=read(sock,buf1,sizeof(buf1)))<=0) {
#endif
#endif
#ifdef DEBUG
                         printf("Connection to NETUSE server was dropped by server.\n");
#endif
                         return(-2);
                    }
                    else bufptr=0;
#ifdef DEBUG
                    printf("soread() success in netuseTCPReceiveMessage()...\n",rv);
#endif
               }
               else {
#ifdef DEBUG
                    printf("select() timed out in netuseTCPReceiveMessage().\n",rv);
#endif
                    return(RV_nTIMEOUT);
               }
          }
     } while (!done);
     if (strlen(msg) && (msg[strlen(msg)-1]=='\n'))
          msg[strlen(msg)-1]='\0';
     if (strlen(msg) && (msg[strlen(msg)-1]=='\r'))
          msg[strlen(msg)-1]='\0';
#ifdef DEBUG
     printf("RECV: \"%s\"\n",msg);
#endif
     return(0);
}


