/******************************************************************************
 NETADMIN.C - NETUSE Administration Client Library Routines

 The functions included here are for making administrative server requests.
 They are fairly powerful, and could be used to deny service to other clients,
 and are included in the library out of pure naivity (and because it wouldn't
 make any difference if they weren't).

 Lee Liming, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#ifdef IBMTCPIP
#include <types.h>
#endif
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include "network.h"
#include "protocol.h"
#include "netuse.h"


/******************************************************************************
 int netuseAddName(char *name,u_short type,u_short model)

 This function adds a host given by name (internet hostname or an ASCII
 equivalent of the IP address) to the server's host list.  If no IP address
 for the host can be found, RV_NOMATCH is returned.  All other errors are
 passed directly back to the caller.  The host's vendor type should be given
 by the type parameter.
******************************************************************************/

#ifdef __STDC__
int netuseAddName(char *name,u_char type,u_char model)
#else
int netuseAddName(name,type,model)
char *name;
u_char type,model;
#endif
{
     USEREC msg;
     int rv;

     msg.opcode=OP_ADDHOST;
     msg.machine=type;
     msg.model=model;
     if (!(msg.ipaddr=netGetAddress(name))) return(RV_NOMATCH);
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if ((rv=packetSend(&msg))!=RV_nOK) return(rv);
     if ((rv=packetReceive(&msg))!=RV_nOK) return(rv);
     else return(msg.retcode);
}


/******************************************************************************
 int netuseAddIP(u_long ip_addr,u_short type)

 This function adds a host to the NETUSE server's host list using the numeric
 IP address instead of the hostname.  Again, the machine type should be given
 in the type parameter.  All errors are passed back to the caller.
******************************************************************************/

#ifdef __STDC__
int netuseAddIP(u_long ip_addr,u_short type)
#else
int netuseAddIP(ip_addr,type)
u_long ip_addr;
u_short type;
#endif
{
     USEREC msg;
     int rv;

     msg.opcode=OP_ADDHOST;
     msg.machine=type;
     msg.ipaddr=ip_addr;
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if ((rv=packetSend(&msg))!=RV_nOK) return(rv);
     if ((rv=packetReceive(&msg))!=RV_nOK) return(rv);
     else return(msg.retcode);
}


/******************************************************************************
 int netuseDelName(char *name)

 The host represented by the hostname given in name (or the IP address in text
 format in name) is removed from the NETUSE server's host list.
******************************************************************************/

#ifdef __STDC__
int netuseDelName(char *name)
#else
int netuseDelName(name)
char *name;
#endif
{
     USEREC msg;
     int rv;

     msg.opcode=OP_DELHOST;
     if (!(msg.ipaddr=netGetAddress(name))) return(RV_NOMATCH);
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if ((rv=packetSend(&msg))!=RV_nOK) return(rv);
     if ((rv=packetReceive(&msg))!=RV_nOK) return(rv);
     else return(msg.retcode);
}


/******************************************************************************
 int netuseSaveState(void)

 This function requests that the server save it's hostlist into the static
 host file.
******************************************************************************/

#ifdef __STDC__
int netuseSaveState(void)
#else
int netuseSaveState()
#endif
{
     USEREC msg;
     int rv;

     msg.opcode=OP_SAVESTATE;
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if ((rv=packetSend(&msg))!=RV_nOK) return(rv);
     if ((rv=packetReceive(&msg))!=RV_nOK) return(rv);
     else return(msg.retcode);
}


/******************************************************************************
 int netuseLoadHosts(void)

 This function requests the server to throw away all of the hosts in it's
 hostlist and load a new hostlist from the static host file.
******************************************************************************/

#ifdef __STDC__
int netuseLoadHosts(void)
#else
int netuseLoadHosts()
#endif
{
     USEREC msg;
     int rv;

     msg.opcode=OP_LOADHOSTS;
#if (defined(MSDOS) || defined(OS2))
     msg.uid=0;
#else
     msg.uid=getuid();
#endif
     if ((rv=packetSend(&msg))!=RV_nOK) return(rv);
     if ((rv=packetReceive(&msg))!=RV_nOK) return(rv);
     else return(msg.retcode);
}
