/******************************************************************************
 NETUSE.H - NETUSE Client Library header file

 This header file defines the functions and data types provided by the NETUSE
 client library.  This is the header file which should be used when linking
 with the libNetuse.a library to create a NETUSE client.

 Lee Liming and Michael Neil, The Computer Aided Engineering Network
 The University of Michigan

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#ifndef _NETUSE_INCLUDED_
#define _NETUSE_INCLUDED_

#include <sys/types.h>
#include "config.h"


#ifndef _PROTOCOL_INCLUDED_

/* Machine/vendor types */

#define MACH_DEC           1                                   /* DECstation */
#define MACH_SUN           2                                          /* Sun */
#define MACH_IBM_RS6000    3                                  /* IBM RS/6000 */
#define MACH_APOLLO        4                                       /* Apollo */
#define MACH_HP            5                                       /* HP9000 */

#define MODEL_DS3100       1                          /* Machine model types */
#define MODEL_DS5000200    2
#define MODEL_DS5000120    3
#define MODEL_SUN4         4
#define MODEL_SUN4_65      5
#define MODEL_SUN4_50      6
#define MODEL_RS320        7
#define MODEL_RS520        8
#define MODEL_RS530        9
#define MODEL_RS540       10
#define MODEL_RS730       11
#define MODEL_RS930       12
#define MODEL_AP3000      13
#define MODEL_AP3500      14
#define MODEL_AP4000      15
#define MODEL_AP4500      16
#define MODEL_AP5500      17
#define MODEL_AP10010     18
#define MODEL_AP10020     19
#define MODEL_HP425E      20
#define MODEL_HP425T      21
#define MODEL_HP9000720   22
#define MODEL_SUN3        23
#define MODEL_AP2500      24
#define MODEL_DS5000133   25
#define MODEL_HP9000705   26
#define MODEL_HP9000710   27
#define MODEL_HP9000730   28
#define MODEL_HP9000750   29
#define MODEL_RS220       30
#define MODEL_RS320H      31
#define MODEL_RS340       32
#define MODEL_RS350       33
#define MODEL_RS530H      34
#define MODEL_RS550       35
#define MODEL_RS560       36
#define MODEL_RS950       37

/* Modes for netuseGetList() */

#define MODE_DISP       0x00        /* Normal listing (limited # of machines */
#define MODE_ALL        0x01     /* List ALL hosts (even ones that are down) */
#define MODE_DOWN       0x02                /* List only hosts that are down */
#define MODE_MASK       0x03                      /* Mask for all list modes */
#define MODE_MODELS     0x04 /* List model types instead of last report time */
#define MODE_NET        0x80                /* Network listing mode (add \r) */

#define RV_OK              0     /* Protocol response: All's well            */
#define RV_ACK             1     /* Protocol response: Report acknowledged   */
#define RV_NACK            2     /* Protocol response: Report not ack'ed     */
#define RV_TIMEOUT         3     /* Protocol response: Timeout               */
#define RV_NOMATCH         4     /* Protocol response: Nothing matched       */
#define RV_NOFILE          5     /* Protocol response: Unable to access file */

#define RV_nOK             0     /* Network response: All's well             */
#define RV_nTIMEOUT       -1     /* Network response: Timeout                */
#define RV_nBADSEND       -2     /* Network response: Unable to send packet  */
#define RV_nBADRECV       -3     /* Network response: Unable to receive      */

#endif   /* !defined(_PROTOCOL_INCLUDED_) */


#ifdef __STDC__

int     netuseGetList(u_char machine,u_char model,int mode);
u_long  netGetAddress(char *buf);
char   *netGetHostname(u_long ipaddr,char *buf);
char   *netGetMachType(u_char mtype,char *mtname);
u_char  netStrToMach(char *mtname);
char   *netGetModelType(u_char mtype,char *mtname);
u_char  netStrToModel(char *mtname);
char   *netuseFind(char *name,double l1,double l2,double l3,int users,
                   int console,int tmp,u_char machine,u_char model);
int     netuseAddName(char *name,u_char type,u_char model);
int     netuseAddIP(u_long ip_addr,u_short type);
int     netuseDelName(char *name);
int     netuseSaveState(void);
int     netuseLoadHosts(void);

#else

int     netuseGetList();
u_long  netGetAddress();
char   *netGetHostname();
char   *netGetMachType();
u_char  netStrToMach();
char   *netGetModelType();
u_char  netStrToModel();
char   *netuseFind();
int     netuseAddName();
int     netuseAddIP();
int     netuseDelName();
int     netuseSaveState();
int     netuseLoadHosts();

#endif

#endif  /* _NETUSE_INCLUDED_ */
