/*****************************************************************************
 VARIABLE.H - The variable management module's header file

 Defines: typedef   VARENTRY
          typedef   VARTABLE
          int       varHashName(u_long name);
          VARENTRY *varAllocate(void);
          void      varFreeTable(void);
          void      varSet(VARENTRY *var);
          VARENTRY *varFindMatch(VARENTRY *template);
          VARENTRY *varFind(u_long ip_addr);
          int       varSaveState(void);
          int       varLoadHosts(void);
          void      varInit();
          void      varDumpTable(char *fname);
          void      treeInsert(TREE **tptr,VARENTRY *vptr);
          void      treeBuild(u_char machtype);
          void      treePrintHost(FILE *outf,TREE *tptr);
          void      treeDump(FILE *outf,TREE *tptr,u_char opcode,int *i);
          void      treePrintList(FILE *outf,u_char machtype,u_char opcode);

 Copyright (C) 1990, 1991, 1992 by the Regents of the University of Michigan.

 User agrees to reproduce said copyright notice on all copies of the software
 made by the recipient.  

 All Rights Reserved. Permission is hereby granted for the recipient to make
 copies and use this software for its own internal purposes only. Recipient of
 this software may re-distribute this software outside of their own
 institution. Permission to market this software commercially, to include this
 product as part of a commercial product, or to make a derivative work for
 commercial purposes, is explicitly prohibited.  All other uses are also
 prohibited unless authorized in writing by the Regents of the University of
 Michigan.

 This software is offered without warranty. The Regents of the University of
 Michigan disclaim all warranties, express or implied, including but not
 limited to the implied warranties of merchantability and fitness for any
 particular purpose. In no event shall the Regents of the University of
 Michigan be liable for loss or damage of any kind, including but not limited
 to incidental, indirect, consequential, or special damages. 
******************************************************************************/

#ifndef _VARIABLE_INCLUDED_

#define _VARIABLE_INCLUDED_


#define VAR_ENTRIES   113            /* Number of entries in the hash table */


typedef struct _user {
             char         *name;
             char         *tty;
             struct _user *next;
        } USER,*USERLIST;

typedef struct _varentry {
             char             *name;                 /* The variable's name */
             u_long            ip_addr;                       /* IP address */
             char             *class;                        /* Entry class */
             float             l1,l2,l3;             /* Load average values */
             u_long            tmp;                      /* kb free on /tmp */
             u_char            machine;                  /* Type of machine */
             u_char            model;               /* Machine's model type */
             long              lastused;        /* Time of entry's last use */
             long              logtime;              /* Time of last update */
             int               users;      /* How many users are logged in? */
             int               console;        /* Is anyone on the console? */
             USERLIST          userFirst;               /* Who is logged in */
             struct _varentry *next;    /* Link to next entry in hash chain */
        } VARENTRY;
typedef VARENTRY *VARTABLE[VAR_ENTRIES];         /* The variable hash table */

typedef struct _nodetree {
             VARENTRY         *data;
             struct _nodetree *left,*right;
        } TREE;


#ifdef __STDC__

int       varHashName(u_long name);
USER     *userAllocate(void);
void      userFree(USER *uptr);
void      userFreeList(USER *uptr);
VARENTRY *varAllocate(void);
void      varFree(VARENTRY *vptr);
void      varFreeTable(void);
void      varSet(VARENTRY *var);
VARENTRY *varFindMatch(VARENTRY *template);
VARENTRY *varFind(u_long ip_addr);
void      varDumpTable(char *fname);
void      varDumpTableWithModels(char *fname);
int       varSaveState(void);
int       varLoadHosts(void);
void      varInit(void);
void      treeInsert(TREE **tptr,VARENTRY *vptr);
void      treeBuild(u_char machtype,u_char modeltype);
void      treePrintHost(FILE *outf,TREE *tptr);
void      treeDump(FILE *outf,TREE *tptr,u_char opcode,int *i);
void      treePrintList(FILE *outf,u_char machtype,u_char modeltype,
                        u_char opcode);
void      treePrintHostWithModels(FILE *outf,TREE *tptr);
void      treeDumpWithModels(FILE *outf,TREE *tptr,u_char opcode,int *i);
void      treePrintListWithModels(FILE *outf,u_char machtype,u_char modeltype,
                                  u_char opcode);

#else  /* ifndef __STDC__ */

int       varHashName();
USER     *userAllocate();
void      userFree();
void      userFreeList();
VARENTRY *varAllocate();
void      varFree();
void      varFreeTable();
void      varSet();
VARENTRY *varFindMatch();
VARENTRY *varGetValue();
int       varSaveState();
int       varLoadHosts();
void      varInit();
void      varDumpTable();
void      varDumpTableWithModels();
int       varSaveState();
int       varLoadHosts();
void      varInit();
void      treeInsert();
void      treeBuild();
void      treePrintHost();
void      treeDump();
void      treePrintList();
void      treePrintHostWithModels();
void      treeDumpWithModels();
void      treePrintListWithModels();

#endif

#endif  /* _VARIABLE_INCLUDED_ */
